
/*
   helicon.c
   
   Main driver for helicon.
   
   Author: Mark E. Nash
   Date:   June, 1988
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<graph.h>
#include	<malloc.h>
#include	<io.h>
#include	<conio.h>
#include	<string.h>
#include	"helimacs.h"
#include	"helimess.h"
#include	"helstruc.h"
#include	"heliglob.h"
#include	"helfdef.h"



main(argc, argv)
	int argc;
	char **argv;
/*
   Main driver.
*/
	{
	char	string[81];			/* for input */
	struct grafwindow *ms;			/* for messages */
	FILE *fp;				/* raw data file */
	struct xycoord xyc1, xyc2;		/* for logical coord conv */
	unsigned int numbins;			/* number of bins to use */
	double	ctr;				/* real gp counter */
	int	idm, idi;			/* menu and item id */
	int	done;				/* event loop condition */
	char	key;				/* key in event */


/* initialize */
	if (hel_init() == ERR)
		hel_exit(0);

/* build menus */
	if ((menus = buildmenus()) == NULL)
		hel_exit(0);
		
/* build dialogs */
	if ((dlogtop = buildlogs()) == NULL)
		hel_exit(0);
		
/* get a command line file name */
	if (argc > 1)
		strncpy(tboxes[DATFIL], argv[1], TBOXSIZE);
	
/* open text windows */
/* info */
	xyc1 = _getlogcoord(IWX1, IWY1);
	if ((iw = open_grafwin(TEXTWIN, xyc1.xcoord, xyc1.ycoord, 0, 0, 
			IBRCOLOR, IBKCOLOR, IWCOLS, IWROWS, FALSE)) == NULL)
		hel_exit();

/* display please wait message */
	eputs_window(iw, "Please wait...", IWSCOLOR, TRUE);

/* graph title */
	xyc1 = _getlogcoord(TIWX1, TIWY1);
	if ((tw = open_grafwin(TEXTWIN, xyc1.xcoord, xyc1.ycoord, 0, 0,
			IBRCOLOR, IBKCOLOR, TICOLS, TIROWS, FALSE)) == NULL)
		hel_exit();

/* x-axis label */
	xyc1 = _getlogcoord(XAWX1, XAWY1);
	if ((xw = open_grafwin(TEXTWIN, xyc1.xcoord, xyc1.ycoord, 0, 0,
			IBRCOLOR, IBKCOLOR, XACOLS, XAROWS, FALSE)) == NULL)
		hel_exit();

/* y-axis label */
	xyc1 = _getlogcoord(YAWX1, YAWY1);
	if ((yw = open_grafwin(TEXTWIN, xyc1.xcoord, xyc1.ycoord, 0, 0,
			IBRCOLOR, IBKCOLOR, YACOLS, YAROWS, FALSE)) == NULL)
		hel_exit();

/* parameters */
	xyc1 = _getlogcoord(PWX1, PWY1);
	if ((pw = open_grafwin(TEXTWIN, xyc1.xcoord, xyc1.ycoord, 0, 0, 
			IBRCOLOR, IBKCOLOR, PWCOLS, PWROWS, FALSE)) == NULL)
		hel_exit();

/* erase wait message */
	win_clear(iw, IBKCOLOR);

/* open data window */
	xyc1 = _getlogcoord(DATAWX1, DATAWY1);
	xyc2 = _getlogcoord(DATAWX2, DATAWY2);
	if ((dw = open_grafwin(GRAFWIN, xyc1.xcoord, xyc1.ycoord, xyc2.xcoord, 
	        	       xyc2.ycoord, DATABRCOLOR, DATABKCOLOR, 0, 0, 
			       					TRUE)) == NULL)
		hel_exit();

/* display current file and fit */
	gtwcurpos(iw, 0, 1);
	eputs_window(iw, "Current File:", IWSCOLOR, FALSE);
	gtwcurpos(iw, IWFNCOL, 1);
	clrstr[20] = '\0';
	eputs_window(iw, clrstr, IBKCOLOR, FALSE);
	clrstr[20] = '\200';
	gtwcurpos(iw, IWFNCOL, 1);
	eputs_window(iw, tboxes[DATFIL], IWDCOLOR, FALSE);
	gtwcurpos(iw, 0, 3);
	eputs_window(iw, "Current Fit:", IWSCOLOR, FALSE);

/* display parameters message */
	gtwcurpos(iw, 0, 7);
	eputs_window(iw, "PARAMETERS:", PWSCOLOR, FALSE);

/* draw menus */
	if (menubar(menus) == ERR)
		hel_exit();
		
/* show mouse cursor */
	MOUSE(1, 0, 0, 0);
		
/* read in data if on command line */
	if (argc > 1)
		readatin();
	
/* main event loop */
	done = FALSE;
	do
		{
		switch (GetNextEvent(&key))
			{
			case NULLEVENT:		/* null event */
				break;
				
			case CURUPDATE:		/* update cursor position */
				if (!mouse)	/* mouse auto updates curpos */
					setcpos(menus, key);
				procmenu(&idm, &idi);  /* process menu steps */
				break;
				
			case ALTEVENT:		/* put cursor at first menu */
				break;
				
			case SELECT:		/* mouse down or return key */
				procfunc(&idm, &idi, &done);/* do a function */
				break;
		
			default:
				break;
			}
			
		} while (!done);

/* finalize things */
	close_grafwin(&dw);
	hel_exit();
	}
	
	
	
int procmenu(idm, idi)
	int *idm, *idi;
/*
   procmenu() does cursor in menu processing.
   
   Entry: The menu structure has been set up and the global menus is the top.
          This routine was called from the main event loop.
   
   Exit:  Menu processing has been completed for this loop.
          idm, idi return current menu and item id's, else NULLID's.
*/
	{
/* first see if a menu is already down */
	if ((*idm = chkpop(menus)) == NULLID)
		if ((*idm = chkbar(menus)) != NULLID)
			pulldown(menus, *idm);
		else;

/* a menu is down */
	else
		{
/* if still in menu bar, do nothing */
		if (*idm == chkbar(menus));
		
		else
			if ((*idi = chkmenu(menus, *idm)) != NULLID);

/* a menu is pulled down, but the cursor has moved out of the menu area. */
			else
				{
				letgo(menus, *idm);
				*idi = NULLID;
				
/* check here to see if we have moved into the next menu */
				if ((*idm = chkbar(menus)) != NULLID)
					pulldown(menus, *idm);
				}
		}
			
	return(NOERR);
	}
	
	
	
int procfunc(mid, iid, finis)
	int *mid, *iid;
	int *finis;
/*
   procfunc() initiates the processing of a helicon function.
   
   Entry: mid, iid are the current menu and item id's.
   
   Exit:  The function indicated by mid and iid has been executed.
          mid and iid are reset to NULLID.
	  The globals cxpos and cypos have been set to zero.
          If the quit function was selected, *finis is TRUE.
*/
	{
	int	tmid, tiid;		/* dummy arguments for procmenu() */
	double	cerr;			/* error between data and dist. */
	struct dialog *dlog;		/* dialog pointer */

	
/* blink the item */
	if (*iid != NULLID)
		hilsel(menus, *mid, *iid, TRUE);

/* get rid of menu */
	cxpos = 0;
	cypos = 0;
	tmid = *mid;
	tiid = *iid;
	setcpos(menus, OUTMENU);
	procmenu(&tmid, &tiid);
	
/* now do the function */
	switch (*mid)
		{
		case FILEID:
			switch (*iid)
				{
				case FNID:	/* get input file name */
					if ((dlog = do_dialog(IFNDID)) != 
									NULL);
						{
						gtwcurpos(iw, IWFNCOL, 1);
						clrstr[20] = '\0';
						eputs_window(iw, clrstr, 
							IBKCOLOR, FALSE);
						clrstr[20] = '\200';
						gtwcurpos(iw, IWFNCOL, 1);
						eputs_window(iw, 
						     tboxes[DATFIL], IWDCOLOR,
						     			FALSE);
						readatin();
						win_clear(pw, IBKCOLOR);
						win_clear(dw, BLACK);
						}
					break;

				case SAVID:
					break;

				case PRID:
					egaprnt();
					break;

				case QID:
					*finis = TRUE;
					break;

				default:
					break;
				}
			break;
		
		case ANAID:
			switch (*iid)
				{
				case CONTID:
					win_clear(dw, BLACK);
					plotdat(CONTINUOUS);
					break;
				
				case DISCID:
					win_clear(dw, BLACK);
					plotdat(DISCRETE);
					break;
					
				case COMPID:
					compwin();
					break;
					
				case ASCALID:
					if (autoscale == FALSE)
						{
						autoscale = TRUE;
						toghil(menus, ANAID, ASCALID);
						toghil(menus, ANAID, MSCALID);
						win_clear(dw, BLACK);
						plotdat(CHOOSE);
						}
					break;
				
				case MSCALID:	/* get user-defined scales */
					if (autoscale == TRUE)
						{
						autoscale = FALSE;
						toghil(menus, ANAID, ASCALID);
						toghil(menus, ANAID, MSCALID);
						}
					do_dialog(SCADID);
					win_clear(dw, BLACK);
					plotdat(CHOOSE);
					break;
					
				case CONSTID:	/* get user-def'ed constants */
					do_dialog(CDID);
		/* reset the random number generated if needed */
					if (rseed != (unsigned)atoi(
								tboxes[SEED]))
						{
						rseed = (unsigned)atoi(
								tboxes[SEED]);
						srand(rseed);
						}
					break;
					
				default:
					break;
				}
			break;
			
				
		case FITID:
	/* tell the current fit */
			gtwcurpos(iw, IWFNCOL, 3);
			clrstr[20] = '\0';
			eputs_window(iw, clrstr, IBKCOLOR, FALSE);
			clrstr[20] = '\200';
			gtwcurpos(iw, IWFNCOL, 3);
			eputs_window(iw, fitlist[*iid+1], IWDCOLOR, FALSE);
	/* clear the parameter window */
			win_clear(pw, IBKCOLOR);

			switch (*iid + 1) /* idx into the params arr */
				{
				case RECT:
					if (rectcalc(bintop, params) == ERR)
						break;
	/* check the parameter range */
					if (params[APAR][RECT] >= 
							params[BPAR][RECT])
						{
						sprintf(msg, D_BADPR2, 
					fitlist[RECT], "b", params[BPAR][PAR],
					"a", params[APAR][RECT]);
						err_window(msg);
						break;
						}
					if (win_func(dw, RECT, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][RECT]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][RECT]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][RECT]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, RECT, params);
					params[MSE][RECT] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case TRI:
					if (tricalc(bintop, params) == ERR)
						break;
					if (win_func(dw, TRI, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][TRI]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][TRI]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "m = %-g",
							params[MPAR][TRI]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][TRI]);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, TRI, params);
					params[MSE][TRI] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+8);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case NOR:
					if (norcalc(bintop, params) == ERR)
						break;
					if (win_func(dw, NOR, params, FUNCOLOR)
								 == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "u = %-g",
							params[NMEAN][NOR]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "s = %-g",
							params[NSDEV][NOR]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][NOR]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, NOR, params);
					params[MSE][NOR] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case GAM:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
					if (gamcalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[CPAR][GAM] <= 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[GAM], "C", params[CPAR][GAM], "0.0");
						err_window(msg);
						break;
						}
					if (params[RPAR][GAM] <= 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[GAM], "r", params[RPAR][GAM], "0.0");
						err_window(msg);
						break;
						}
						
					if (win_func(dw, GAM, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "C = %-g",
							params[CPAR][GAM]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "r = %-g",
							params[RPAR][GAM]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][GAM]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, GAM, params);
					params[MSE][GAM] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case PAR:
					if (parcalc(bintop, params) == ERR)
						break;
	/* check the parameter range */
					if (params[BPAR][PAR] <= 1.0)
						{
						sprintf(msg, D_BADPAR, 
				fitlist[PAR], "b", params[BPAR][PAR], "1.0");
						err_window(msg);
						break;
						}
						
					if (params[APAR][PAR] <= 2.0)
						{
						sprintf(msg, D_BADPAR, 
				fitlist[PAR], "a", params[APAR][PAR], "2.0");
						err_window(msg);
						break;
						}
						
					if (bintop->xmin < params[BPAR][PAR])
						err_window(M_IGPT);
						
					if (win_func(dw, PAR, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][PAR]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][PAR]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][PAR]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, PAR, params);
					params[MSE][PAR] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case FDIST:
					if (fcalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[FNPAR][FDIST] < 4.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[FDIST], "n", params[FNPAR][FDIST], "4.0");
						err_window(msg);
						break;
						}
						
					if (params[FMPAR][FDIST] <= 1.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[FDIST], "m", params[FMPAR][FDIST], "1.0");
						err_window(msg);
						break;
						}
						
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
					if (win_func(dw, FDIST, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "m = %-g",
							params[FMPAR][FDIST]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "n = %-g",
							params[FNPAR][FDIST]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][FDIST]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, FDIST, params);
					params[MSE][FDIST] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case BETA:
					if (betacalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[APAR][BETA] < 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[BETA], "a", params[APAR][BETA], "0.0");
						err_window(msg);
						break;
						}
						
					if (params[BPAR][BETA] < 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[BETA], "b", params[BPAR][BETA], "0.0");
						err_window(msg);
						break;
						}
	/* check the range */
					if (bintop->xmin < 0.0 || 
							bintop->xmax > 1.0)
						err_window(M_IGPT);
						
					if (win_func(dw, BETA, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][BETA]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][BETA]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][BETA]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, BETA, params);
					params[MSE][BETA] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case LOGNOR:
	/* check the range */
					if (bintop->xmin <= 0.0)
						err_window(M_IGPT);
						
					if (lnorcalc(bintop, params) == ERR)
						break;

					if (win_func(dw, LOGNOR, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "m = %-g",
							params[LMPAR][LOGNOR]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "r = %-g",
							params[LRPAR][LOGNOR]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][LOGNOR]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, LOGNOR, params);
					params[MSE][LOGNOR] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case EXPON:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
					if (excalc(bintop, params) == ERR)
						break;

	/* check the parameters */
					if (params[CPAR][EXPON] <= 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[EXPON], "C", params[CPAR][EXPON], "0.0");
						err_window(msg);
						break;
						}

					if (win_func(dw, EXPON, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "C = %-g",
							params[CPAR][EXPON]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][EXPON]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, EXPON, params);
					params[MSE][EXPON] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case DEXPON:
					if (dexcalc(bintop, params) == ERR)
						break;

					if (win_func(dw, DEXPON, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][DEXPON]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][DEXPON]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][DEXPON]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, DEXPON, params);
					params[MSE][DEXPON] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case GUM:
					if (gumcalc(bintop, params) == ERR)
						break;

					if (win_func(dw, GUM, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][GUM]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][GUM]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][GUM]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, GUM, params);
					params[MSE][GUM] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case CHI:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
					if (chicalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[KPAR][CHI] < 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[CHI], "k", params[KPAR][CHI], "0.0");
						err_window(msg);
						break;
						}
						
					if (win_func(dw, CHI, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "k = %-g",
							params[KPAR][CHI]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][CHI]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, CHI, params);
					params[MSE][CHI] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case LOGIS:
					if (logcalc(bintop, params) == ERR)
						break;

					if (win_func(dw, LOGIS, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][LOGIS]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][LOGIS]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][LOGIS]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, LOGIS, params);
					params[MSE][LOGIS] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case TDIST:
					if (tcalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[KPAR][TDIST] < 2.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[TDIST], "k", params[KPAR][TDIST], "2.0");
						err_window(msg);
						break;
						}

					if (win_func(dw, TDIST, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "k = %-g",
							params[KPAR][TDIST]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][TDIST]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, TDIST, params);
					params[MSE][TDIST] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case WEI:
	/* check the range */
					if (bintop->xmin <= 0.0)
						err_window(M_IGPT);
						
					if (weicalc(bintop, params) == ERR)
						break;
					if (win_func(dw, WEI, params, FUNCOLOR)
								 == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "a = %-g",
							params[APAR][WEI]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "b = %-g",
							params[BPAR][WEI]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][WEI]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, WEI, params);
					params[MSE][WEI] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case BINOM:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
	/* see if calculation is possible */
					if (bintop->var >= bintop->mean)
						{
						err_window(M_BADB);
						break;
						}

					if (bincalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[PPAR][BINOM] <= 0.0 ||
						params[PPAR][BINOM] >= 1.0)
						{
						sprintf(msg, D_BADRAN, 
				     fitlist[BINOM], "p", params[PPAR][BINOM]);
						err_window(msg);
						break;
						}
						
					if (params[NPAR][BINOM] <= 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[BINOM], "n", params[NPAR][BINOM], "0.0");
						err_window(msg);
						break;
						}

					if (win_func(dw, BINOM, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "n = %-g",
							params[NPAR][BINOM]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "p = %-g",
							params[PPAR][BINOM]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "q = %-g",
						1.0-params[PPAR][BINOM]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][BINOM]);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, BINOM, params);
					params[MSE][BINOM] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+8);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case POI:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);
						
					if (poicalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[CPAR][POI] <= 0.0)
						{
						sprintf(msg, D_BADPAR, 
			     fitlist[POI], "C", params[CPAR][POI], "0.0");
						err_window(msg);
						break;
						}
						
					if (win_func(dw, POI, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "C = %-g",
							params[CPAR][POI]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][POI]);
					gtwcurpos(pw, 0, SPAROW+4);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, POI, params);
					params[MSE][POI] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				case GEO:
	/* check the range */
					if (bintop->xmin < 0.0)
						err_window(M_IGPT);

					if (geocalc(bintop, params) == ERR)
						break;
	/* check the parameters */
					if (params[PPAR][GEO] <= 0.0 ||
						params[PPAR][GEO] >= 1.0)
						{
						sprintf(msg, D_BADRAN, 
				     fitlist[GEO], "p", params[PPAR][GEO]);
						err_window(msg);
						break;
						}
						
					if (win_func(dw, GEO, params, 
							FUNCOLOR) == ERR)
						break;
	/* display the parameters */
					sprintf(msg, "p = %-g",
							params[PPAR][GEO]);
					gtwcurpos(pw, 0, SPAROW);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "q = %-g",
							1.0-params[PPAR][GEO]);
					gtwcurpos(pw, 0, SPAROW+2);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					sprintf(msg, "mag = %-g",
							params[MAG][GEO]);
					gtwcurpos(pw, 0, SPAROW+6);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
				cerr = mse(bintop->start, GEO, params);
					params[MSE][GEO] = cerr;
					sprintf(msg, "mse = %-g", cerr);
					gtwcurpos(pw, 0, SPAROW+8);
					eputs_window(pw, msg, PWDCOLOR, FALSE);
					break;
					
				default:
					break;
				}

		/* show mean and variance */
			if (bintop != NULL)
				{
				gtwcurpos(pw, 0, PWROWS-2);
				sprintf(msg, "mean = %-g", bintop->mean);
				eputs_window(pw, msg, PWDCOLOR, FALSE);
				gtwcurpos(pw, 0, PWROWS-1);
				sprintf(msg, "variance = %-g", bintop->var);
				eputs_window(pw, msg, PWDCOLOR, FALSE);
				}
			break;
			
		case BUILDID:
			switch (*iid)
				{
				case ESTID:
					bdist();
					break;
				default:
					break;
				}
			break;
		
		case DSPID:
			switch (*iid)
				{
				case TID:
					do_dialog(TITLID);
					dislabs();
					break;
				default:
					break;
				}
			break;
			
		case UTILID:
			switch (*iid)
				{
				case PRTYID:
					do_dialog(PTYDID);
					prtyp = printyp();   /* get the type */
					break;
				case HINFO:
					about();
					break;
				default:
					break;
				}
			break;
			
		default:
			break;
		}

/* set menu selections to NULLID before return */
	*mid = NULLID;
	*iid = NULLID;
	return(NOERR);
	}
	
	
	
int memupdt(size, af)
	unsigned long size;
	int af;
/*
   memupdt() updates the memory monitor window.
   
   Entry: A memory allocation or free has been called.
          The globals mfree and mall are current.
	  size is the size of the memory block that was allocated or freed.
	  af = ALLOC for allocation, FRE for free.
	       
   Exit:  The free and allocated memory indicators have been updated
		in the memory monitor window.
*/
	{
	double	xbeg, xend, delta1, delta2;	/* for memory bar */
	short	bcolor;				/* color of bar to extend */
	char	mmesg[MESSLEN+1];		/* for making strings */

	
/* update globals */
	delta1 = 0.0;
	delta2 = 0.0;
	if (af == ALLOC)
		{
		mfree -= size;
		mall += size;
		delta1 = ((double)size) / 1000.0;
		bcolor = MALCOLOR;
		}
	else
		{
		mfree += size;
		mall -= size;
		delta2 = ((double)size) / 1000.0;
		bcolor = MFCOLOR;
		}

	return(NOERR);
	}
	
	
	
int about()
/*
   about() displays the info about this program.
      
   Entry: The "about" command has been selected.
   
   Exit:  If no errors, the message has been displayed, and the user 
   		has responded.
*/
	{
	struct grafwindow *ab;			/* error window */
	register unsigned int ctr;		/* jeep counter */
	struct xycoord xy;			/* coord conv */
	int	done;				/* mouse loop */
	
	
	xy = _getlogcoord(ABX1, ABY1);
	if ((ab = open_grafwin(TEXTWIN, xy.xcoord, xy.ycoord, 0, 0, ABBORCOLOR,
			   	ABBACKCOLOR, ABCOLSIZE, 11, TRUE)) == NULL)
		return(ERR);
	
	ctr = 0;
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, "Helicon Distribution Fitting Tool", ABTICOLOR, TRUE);
	sprintf(msg, "Version 0.0 "),
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, msg, ABTICOLOR, TRUE);
	sprintf(msg, "Serial Number: %s", snumber);
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, msg, ABTICOLOR, TRUE);
	ctr++;
	gtwcurpos(ab, 0, ctr++);
	sprintf(msg, "Written by %s", author);
	eputs_window(ab, msg, ABINCOLOR, TRUE);
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, "Micro Analysis and Design", ABINCOLOR, TRUE);
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, "3300 Mitchell Lane, Suite 175", ABINCOLOR, TRUE);
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, "Boulder, Colorado  80301", ABINCOLOR, TRUE);
	gtwcurpos(ab, 0, ctr++);
	eputs_window(ab, "(303) 442-6947", ABINCOLOR, TRUE);
	ctr++;
	
/* Display prompt message */
	gtwcurpos(ab, 0, ctr);
	if (mouse)
		eputs_window(ab, MERRPROMP, EPRCOLOR, TRUE);
	else
		eputs_window(ab, ERRPROMP, EPRCOLOR, TRUE);
	
	if (mouse)
		{
/* check the mouse status */
		done = FALSE;
		while (!done)
			{
			MOUSE(3, 0, 0, 0);
			xy = _getlogcoord(m3, m4);
			cxpos = xy.xcoord;
			cypos = xy.ycoord;
			if (m2 != 0)
				if (curnrgn(&(ab->wbox)))
					done = TRUE;
			}
		}
/* wait for a return character */
	else
		while (getch() != RETURN);
	
	close_grafwin(&ab);
	return(NOERR);
	}
